@@ -43,10 +43,13 @@ for a number of tasks:
 import pkg_resources
 from flask import Flask, has_app_context, url_for
 from flask_sqlalchemy import SQLAlchemy
+from sqlalchemy.pool import StaticPool
 from logging import getLogger
+import gi  # noqa
+gi.require_version("Modulemd", "2.0")  # noqa
+from gi.repository import Modulemd  # noqa
 
-from module_build_service.logger import (
-    init_logging, ModuleBuildLogs, level_flags)
+from module_build_service.logger import init_logging, ModuleBuildLogs, level_flags, MBSLogger
 
 from module_build_service.errors import (
     ValidationError, Unauthorized, UnprocessableEntity, Conflict, NotFound,
@@ -55,15 +58,38 @@ from module_build_service.config import init_config
 from module_build_service.proxy import ReverseProxy
 
 try:
-    version = pkg_resources.get_distribution('module-build-service').version
+    version = pkg_resources.get_distribution("module-build-service").version
 except pkg_resources.DistributionNotFound:
-    version = 'unknown'
+    version = "unknown"
+api_version = 2
 
 app = Flask(__name__)
 app.wsgi_app = ReverseProxy(app.wsgi_app)
 
 conf = init_config(app)
-db = SQLAlchemy(app)
+
+
+class MBSSQLAlchemy(SQLAlchemy):
+    """
+    Inherits from SQLAlchemy and if SQLite in-memory database is used,
+    sets the driver options so multiple threads can share the same database.
+
+    This is used *only* during tests to make them faster.
+    """
+
+    def apply_driver_hacks(self, app, info, options):
+        if info.drivername == "sqlite" and info.database in (None, "", ":memory:"):
+            options["poolclass"] = StaticPool
+            options["connect_args"] = {"check_same_thread": False}
+            try:
+                del options["pool_size"]
+            except KeyError:
+                pass
+
+        super(MBSSQLAlchemy, self).apply_driver_hacks(app, info, options)
+
+
+db = MBSSQLAlchemy(app)
 
 
 def create_app(debug=False, verbose=False, quiet=False):
@@ -81,54 +107,56 @@ def create_app(debug=False, verbose=False, quiet=False):
 
 def load_views():
     from module_build_service import views
+
     assert views
 
 
 @app.errorhandler(ValidationError)
 def validationerror_error(e):
     """Flask error handler for ValidationError exceptions"""
-    return json_error(400, 'Bad Request', e.args[0])
+    return json_error(400, "Bad Request", str(e))
 
 
 @app.errorhandler(Unauthorized)
 def unauthorized_error(e):
     """Flask error handler for NotAuthorized exceptions"""
-    return json_error(401, 'Unauthorized', e.args[0])
+    return json_error(401, "Unauthorized", str(e))
 
 
 @app.errorhandler(Forbidden)
 def forbidden_error(e):
     """Flask error handler for Forbidden exceptions"""
-    return json_error(403, 'Forbidden', e.args[0])
+    return json_error(403, "Forbidden", str(e))
 
 
 @app.errorhandler(RuntimeError)
 def runtimeerror_error(e):
     """Flask error handler for RuntimeError exceptions"""
-    return json_error(500, 'Internal Server Error', e.args[0])
+    log.exception("RuntimeError exception raised")
+    return json_error(500, "Internal Server Error", str(e))
 
 
 @app.errorhandler(UnprocessableEntity)
 def unprocessableentity_error(e):
     """Flask error handler for UnprocessableEntity exceptions"""
-    return json_error(422, 'Unprocessable Entity', e.args[0])
+    return json_error(422, "Unprocessable Entity", str(e))
 
 
 @app.errorhandler(Conflict)
 def conflict_error(e):
     """Flask error handler for Conflict exceptions"""
-    return json_error(409, 'Conflict', e.args[0])
+    return json_error(409, "Conflict", str(e))
 
 
 @app.errorhandler(NotFound)
 def notfound_error(e):
     """Flask error handler for Conflict exceptions"""
-    return json_error(404, 'Not Found', e.args[0])
+    return json_error(404, "Not Found", str(e))
 
 
 init_logging(conf)
-log = getLogger(__name__)
-build_logs = ModuleBuildLogs(conf.build_logs_dir)
+log = MBSLogger()
+build_logs = ModuleBuildLogs(conf.build_logs_dir, conf.build_logs_name_format, conf.log_level)
 
 
 def get_url_for(*args, **kwargs):
@@ -140,11 +168,13 @@ def get_url_for(*args, **kwargs):
 
     # Localhost is right URL only when the scheduler runs on the same
     # system as the web views.
-    app.config['SERVER_NAME'] = 'localhost'
+    app.config["SERVER_NAME"] = "localhost"
     with app.app_context():
-        log.debug("WARNING: get_url_for() has been called without the Flask "
-                  "app_context. That can lead to SQLAlchemy errors caused by "
-                  "multiple session being used in the same time.")
+        log.debug(
+            "WARNING: get_url_for() has been called without the Flask "
+            "app_context. That can lead to SQLAlchemy errors caused by "
+            "multiple session being used in the same time."
+        )
         return url_for(*args, **kwargs)
 
 
