@@ -26,10 +26,10 @@ import logging
 import os
 import koji
 import kobo.rpmlib
-import modulemd
 import pipes
 import platform
 import re
+import subprocess
 import threading
 
 from module_build_service import conf, log
@@ -38,24 +38,36 @@ import module_build_service.utils
 import module_build_service.scheduler
 import module_build_service.scheduler.consumer
 
-from base import GenericBuilder
-from utils import (
+from module_build_service.builder.base import GenericBuilder
+from module_build_service.builder.utils import (
     create_local_repo_from_koji_tag,
     execute_cmd,
     find_srpm,
+    get_koji_config,
 )
-from KojiModuleBuilder import KojiModuleBuilder
-from module_build_service.models import ModuleBuild
+from module_build_service.utils.general import mmd_to_str
+from module_build_service.builder.KojiModuleBuilder import KojiModuleBuilder
+
+from module_build_service import models
 
 logging.basicConfig(level=logging.DEBUG)
 
 
-class MockModuleBuilder(GenericBuilder):
+def detect_arch():
     """
-    See http://blog.samalik.com/copr-in-the-modularity-world/
-    especially section "Building a stack"
+    Helper method to detect the local host architecture. Fallbacks to `conf.arch_fallback`.
     """
+    if conf.arch_autodetect:
+        arch_detected = platform.machine()
+        if arch_detected:
+            return arch_detected
 
+        log.warning("Couldn't determine machine arch. Falling back to configured arch.")
+
+    return conf.arch_fallback
+
+
+class MockModuleBuilder(GenericBuilder):
     backend = "mock"
     # Global build_id/task_id we increment when new build is executed.
     _build_id_lock = threading.Lock()
@@ -71,8 +83,7 @@ class MockModuleBuilder(GenericBuilder):
         except IOError:
             pass
     else:
-        raise IOError("None of {} mock config files found."
-                      .format(conf.mock_config_file))
+        raise IOError("None of {} mock config files found.".format(conf.mock_config_file))
 
     # Load yum config file template
     for cf in conf.yum_config_file:
@@ -83,29 +94,23 @@ class MockModuleBuilder(GenericBuilder):
         except IOError:
             pass
     else:
-        raise IOError("None of {} yum config files found."
-                      .format(conf.yum_config_file))
+        raise IOError("None of {} yum config files found.".format(conf.yum_config_file))
 
-    @module_build_service.utils.validate_koji_tag('tag_name')
-    def __init__(self, owner, module, config, tag_name, components):
+    @module_build_service.utils.validate_koji_tag("tag_name")
+    def __init__(self, db_session, owner, module, config, tag_name, components):
+        self.db_session = db_session
         self.module_str = module.name
+        self.module = module
         self.tag_name = tag_name
         self.config = config
         self.groups = []
+        self.enabled_modules = []
+        self.releasever = module_build_service.utils.get_local_releasever()
         self.yum_conf = MockModuleBuilder.yum_config_template
         self.koji_session = None
 
         # Auto-detect arch (if possible) or fallback to the configured one
-        if conf.arch_autodetect:
-            arch_detected = platform.machine()
-            if arch_detected:
-                self.arch = arch_detected
-            else:
-                log.warning("Couldn't determine machine arch. Falling back "
-                            "to configured arch.")
-                self.arch = conf.arch_fallback
-        else:
-            self.arch = conf.arch_fallback
+        self.arch = detect_arch()
         log.info("Machine arch setting: {}".format(self.arch))
 
         # Create main directory for this tag
@@ -144,15 +149,26 @@ class MockModuleBuilder(GenericBuilder):
             for name in os.listdir(self.configdir):
                 os.remove(os.path.join(self.configdir, name))
 
-        log.info("MockModuleBuilder initialized, tag_name=%s, tag_dir=%s" %
-                 (tag_name, self.tag_dir))
+        log.info(
+            "MockModuleBuilder initialized, tag_name=%s, tag_dir=%s" % (tag_name, self.tag_dir))
 
     @property
     def module_build_tag(self):
         # Workaround koji specific code in modules.py
         return {"name": self.tag_name}
 
-    def _createrepo(self):
+    @classmethod
+    def get_module_build_arches(cls, module):
+        """
+        :param ModuleBuild module: Get the list of architectures associated with
+            the module build in the build system.
+        :return: list of architectures
+        """
+        # Return local architecture, because all the modules built locally are built
+        # just against this architecture.
+        return [detect_arch()]
+
+    def _createrepo(self, include_module_yaml=False):
         """
         Creates the repository using "createrepo_c" command in the resultsdir.
         """
@@ -172,30 +188,54 @@ class MockModuleBuilder(GenericBuilder):
         pkglist_f = open(pkglist, "w")
 
         # Generate the mmd the same way as pungi does.
-        m1 = ModuleBuild.query.filter(ModuleBuild.name == self.module_str).one()
-        m1_mmd = m1.mmd()
-        for rpm in os.listdir(self.resultsdir):
-            if not rpm.endswith(".rpm"):
-                continue
+        m1_mmd = self.module.mmd()
+        artifacts = set()
 
-            if m1.last_batch_id() == m1.batch:
-                # If RPM is filtered-out, do not add it to artifacts list.
-                nvr = kobo.rpmlib.parse_nvr(rpm)
-                if nvr["name"] in m1_mmd.filter.rpms:
-                    continue
+        rpm_files = [f for f in os.listdir(self.resultsdir) if f.endswith(".rpm")]
 
-            pkglist_f.write(rpm + '\n')
-            rpm = rpm[:-len(".rpm")]
-            m1_mmd.artifacts.add_rpm(str(rpm))
+        if rpm_files:
+            output = subprocess.check_output(
+                [
+                    "rpm",
+                    "--queryformat",
+                    "%{NAME} %{EPOCHNUM} %{VERSION} %{RELEASE} %{ARCH}\n",
+                    "-qp",
+                ]
+                + rpm_files,
+                cwd=self.resultsdir,
+                universal_newlines=True,
+            )
+            nevras = output.strip().split("\n")
+            if len(nevras) != len(rpm_files):
+                raise RuntimeError("rpm -qp returned an unexpected number of lines")
+
+            for rpm_file, nevra in zip(rpm_files, nevras):
+                name, epoch, version, release, arch = nevra.split()
+
+                if self.module.last_batch_id() == self.module.batch:
+                    # If RPM is filtered-out, do not add it to artifacts list.
+                    if name in m1_mmd.get_rpm_filters():
+                        continue
+
+                pkglist_f.write(rpm_file + "\n")
+                artifacts.add("{}-{}:{}-{}.{}".format(name, epoch, version, release, arch))
 
         pkglist_f.close()
+        # There is no way to replace the RPM artifacts, so remove any extra RPM artifacts
+        for artifact_to_remove in (set(m1_mmd.get_rpm_artifacts()) - artifacts):
+            m1_mmd.remove_rpm_artifact(artifact_to_remove)
+        for artifact_to_add in (artifacts - set(m1_mmd.get_rpm_artifacts())):
+            m1_mmd.add_rpm_artifact(artifact_to_add)
 
-        mmd_path = os.path.join(path, "modules.yaml")
-        modulemd.dump_all(mmd_path, [m1_mmd])
+        # Generate repo.
+        execute_cmd(["/usr/bin/createrepo_c", "--pkglist", pkglist, path])
 
-        # Generate repo and inject modules.yaml there.
-        execute_cmd(['/usr/bin/createrepo_c', '--pkglist', pkglist, path])
-        execute_cmd(['/usr/bin/modifyrepo_c', '--mdtype=modules', mmd_path, repodata_path])
+        # ...and inject modules.yaml there if asked.
+        if include_module_yaml:
+            mmd_path = os.path.join(path, "modules.yaml")
+            with open(mmd_path, "w") as f:
+                f.write(mmd_to_str(m1_mmd))
+            execute_cmd(["/usr/bin/modifyrepo_c", "--mdtype=modules", mmd_path, repodata_path])
 
     def _add_repo(self, name, baseurl, extra=""):
         """
@@ -208,6 +248,14 @@ class MockModuleBuilder(GenericBuilder):
         self.yum_conf += extra
         self.yum_conf += "enabled=1\n\n"
 
+    def _add_repo_from_path(self, path):
+        """
+        Adds repository stored in `path` to Mock config file. Call _write_mock_config() to
+        actually write the config file to filesystem.
+        """
+        with open(path) as fd:
+            self.yum_conf += fd.read()
+
     def _load_mock_config(self):
         """
         Loads the variables which are generated only during the first
@@ -223,16 +271,18 @@ class MockModuleBuilder(GenericBuilder):
 
         with MockModuleBuilder._config_lock:
             infile = os.path.join(self.configdir, "mock.cfg")
-            with open(infile, 'r') as f:
+            with open(infile, "r") as f:
                 # This looks scary, but it is the way how mock itself loads the
                 # config file ...
                 config_opts = {}
-                code = compile(f.read(), infile, 'exec')
+                code = compile(f.read(), infile, "exec")
                 # pylint: disable=exec-used
                 exec(code)
 
                 self.groups = config_opts["chroot_setup_cmd"].split(" ")[1:]
-                self.yum_conf = config_opts['yum.conf']
+                self.yum_conf = config_opts["yum.conf"]
+                self.enabled_modules = config_opts["module_enable"]
+                self.releasever = config_opts["releasever"]
 
     def _write_mock_config(self):
         """
@@ -241,22 +291,24 @@ class MockModuleBuilder(GenericBuilder):
 
         with MockModuleBuilder._config_lock:
             config = str(MockModuleBuilder.mock_config_template)
-            config = config.replace("$root", "%s-%s" % (self.tag_name,
-                                                        str(threading.current_thread().name)))
+            config = config.replace(
+                "$root", "%s-%s" % (self.tag_name, str(threading.current_thread().name)))
             config = config.replace("$arch", self.arch)
             config = config.replace("$group", " ".join(self.groups))
             config = config.replace("$yum_conf", self.yum_conf)
+            config = config.replace("$enabled_modules", str(self.enabled_modules))
+            config = config.replace("$releasever", str(self.releasever))
 
             # We write the most recent config to "mock.cfg", so thread-related
             # configs can be later (re-)generated from it using _load_mock_config.
             outfile = os.path.join(self.configdir, "mock.cfg")
-            with open(outfile, 'w') as f:
+            with open(outfile, "w") as f:
                 f.write(config)
 
             # Write the config to thread-related configuration file.
-            outfile = os.path.join(self.configdir, "mock-%s.cfg" %
-                                   str(threading.current_thread().name))
-            with open(outfile, 'w') as f:
+            outfile = os.path.join(
+                self.configdir, "mock-%s.cfg" % str(threading.current_thread().name))
+            with open(outfile, "w") as f:
                 f.write(config)
 
     def buildroot_connect(self, groups):
@@ -291,29 +343,61 @@ class MockModuleBuilder(GenericBuilder):
                 self._write_mock_config()
 
         from module_build_service.scheduler.consumer import fake_repo_done_message
+
         fake_repo_done_message(self.tag_name)
 
     def tag_artifacts(self, artifacts):
         pass
 
     def buildroot_add_repos(self, dependencies):
-        # TODO: We support only dependencies from Koji here. This should be
-        # extended to Copr in the future.
         self._load_mock_config()
-        for tag in dependencies:
-            # If tag starts with mock_resultdir, it means it is path to local
+        for source, mmds in dependencies.items():
+            # If source starts with mock_resultdir, it means it is path to local
             # module build repository.
-            if tag.startswith(conf.mock_resultsdir):
-                repo_name = os.path.basename(tag)
+            if source.startswith(conf.mock_resultsdir):
+                repo_name = os.path.basename(source)
                 if repo_name.startswith("module-"):
                     repo_name = repo_name[7:]
-                repo_dir = tag
+                repo_dir = source
+                baseurl = "file://" + repo_dir
+            # If source starts with "repofile://", it is path to local /etc/yum.repos.d
+            # repo file.
+            elif source.startswith("repofile://"):
+                # For the base module, we want to include all the `conf.base_module_repofiles`.
+                if len(mmds) == 1 and mmds[0].get_module_name() in conf.base_module_names:
+                    for repofile in conf.base_module_repofiles:
+                        self._add_repo_from_path(repofile)
+                    # Also set the platform_id.
+                    mmd = mmds[0]
+                    self.yum_conf = self.yum_conf.replace(
+                        "$module_platform_id",
+                        "%s:%s" % (mmd.get_module_name(), mmd.get_stream_name())
+                    )
+                else:
+                    # Add repositories defined in repofile to mock config.
+                    repofile = source[len("repofile://"):]
+                    self._add_repo_from_path(repofile)
+                    # Enabled all the modular dependencies by default in Mock.
+                    for mmd in mmds:
+                        self.enabled_modules.append(
+                            "%s:%s" % (mmd.get_module_name(), mmd.get_stream_name()))
+                continue
             else:
-                repo_name = tag
-                repo_dir = os.path.join(self.config.cache_dir, "koji_tags", tag)
-                create_local_repo_from_koji_tag(self.config, tag, repo_dir,
-                                                [self.arch, "noarch"])
-            baseurl = "file://" + repo_dir
+                repo_name = tag = source
+                koji_config = get_koji_config(self.config)
+                koji_session = koji.ClientSession(koji_config.server, opts=koji_config)
+                repo = koji_session.getRepo(repo_name)
+                if repo:
+                    baseurl = koji.PathInfo(topdir=koji_config.topurl).repo(repo["id"], repo_name)
+                    baseurl = "{0}/{1}/".format(baseurl, self.arch)
+                else:
+                    repo_dir = os.path.join(self.config.cache_dir, "koji_tags", tag)
+                    create_local_repo_from_koji_tag(
+                        self.config, tag, repo_dir, [self.arch, "noarch"])
+                    baseurl = "file://" + repo_dir
+                # Check to see if there are any external repos tied to the tag
+                for ext_repo in koji_session.getTagExternalRepos(repo_name):
+                    self._add_repo(ext_repo["external_repo_name"], ext_repo["url"])
             self._add_repo(repo_name, baseurl)
         self._write_mock_config()
 
@@ -326,13 +410,13 @@ class MockModuleBuilder(GenericBuilder):
         # build_id=1 and task_id=1 are OK here, because we are building just
         # one RPM at the time.
         msg = module_build_service.messaging.KojiBuildChange(
-            msg_id='a faked internal message',
+            msg_id="a faked internal message",
             build_id=build_id,
             task_id=build_id,
             build_name=nvr["name"],
             build_new_state=state,
             build_release=nvr["release"],
-            build_version=nvr["version"]
+            build_version=nvr["version"],
         )
         module_build_service.scheduler.consumer.work_queue_put(msg)
 
@@ -355,7 +439,7 @@ class MockModuleBuilder(GenericBuilder):
                 os.remove(log_path)
 
             # Remove other files containing useless information
-            elif logf.endswith('-srpm-stdout.log'):
+            elif logf.endswith("-srpm-stdout.log"):
                 with open(log_path) as f:
                     data = f.read(4096)
                     if re.match("Downloading [^\n]*\n\n\nWrote: [^\n]", data):
@@ -365,24 +449,27 @@ class MockModuleBuilder(GenericBuilder):
         """
         Builds the artifact from the SRPM.
         """
-        state = koji.BUILD_STATES['BUILDING']
+        state = koji.BUILD_STATES["BUILDING"]
 
         # Use the mock config associated with this thread.
-        mock_config = os.path.join(self.configdir,
-                                   "mock-%s.cfg" % str(threading.current_thread().name))
+        mock_config = os.path.join(
+            self.configdir, "mock-%s.cfg" % str(threading.current_thread().name))
 
         # Open the logs to which we will forward mock stdout/stderr.
-        mock_stdout_log = open(os.path.join(self.resultsdir,
-                                            artifact_name + "-mock-stdout.log"), "w")
-        mock_stderr_log = open(os.path.join(self.resultsdir,
-                                            artifact_name + "-mock-stderr.log"), "w")
+        mock_stdout_log = open(
+            os.path.join(self.resultsdir, artifact_name + "-mock-stdout.log"), "w")
+        mock_stderr_log = open(
+            os.path.join(self.resultsdir, artifact_name + "-mock-stderr.log"), "w")
 
         srpm = artifact_name
         resultsdir = builder.resultsdir
         try:
             # Initialize mock.
-            execute_cmd(["mock", "-v", "-r", mock_config, "--init"],
-                        stdout=mock_stdout_log, stderr=mock_stderr_log)
+            execute_cmd(
+                ["mock", "-v", "-r", mock_config, "--init"],
+                stdout=mock_stdout_log,
+                stderr=mock_stderr_log,
+            )
 
             # Start the build and store results to resultsdir
             builder.build(mock_stdout_log, mock_stderr_log)
@@ -392,23 +479,21 @@ class MockModuleBuilder(GenericBuilder):
             # are put in the scheduler's work queue and are handled
             # by MBS after the build_srpm() method returns and scope gets
             # back to scheduler.main.main() method.
-            state = koji.BUILD_STATES['COMPLETE']
+            state = koji.BUILD_STATES["COMPLETE"]
             self._send_build_change(state, srpm, build_id)
 
-            with open(os.path.join(resultsdir, "status.log"), 'w') as f:
+            with open(os.path.join(resultsdir, "status.log"), "w") as f:
                 f.write("complete\n")
         except Exception as e:
-            log.error("Error while building artifact %s: %s" % (artifact_name,
-                                                                str(e)))
+            log.error("Error while building artifact %s: %s" % (artifact_name, str(e)))
 
             # Emit messages simulating complete build. These messages
             # are put in the scheduler's work queue and are handled
             # by MBS after the build_srpm() method returns and scope gets
             # back to scheduler.main.main() method.
-            state = koji.BUILD_STATES['FAILED']
-            self._send_build_change(state, srpm,
-                                    build_id)
-            with open(os.path.join(resultsdir, "status.log"), 'w') as f:
+            state = koji.BUILD_STATES["FAILED"]
+            self._send_build_change(state, srpm, build_id)
+            with open(os.path.join(resultsdir, "status.log"), "w") as f:
                 f.write("failed\n")
 
         mock_stdout_log.close()
@@ -437,7 +522,7 @@ class MockModuleBuilder(GenericBuilder):
         # already in repository ready to be used. This is not a case for Mock
         # backend in the time we return here.
         reason = "Building %s in Mock" % (artifact_name)
-        return build_id, koji.BUILD_STATES['BUILDING'], reason, None
+        return build_id, koji.BUILD_STATES["BUILDING"], reason, None
 
     def build(self, artifact_name, source):
         log.info("Starting building artifact %s: %s" % (artifact_name, source))
@@ -446,8 +531,8 @@ class MockModuleBuilder(GenericBuilder):
         # generate the thread-specific mock config by writing it to fs again.
         self._load_mock_config()
         self._write_mock_config()
-        mock_config = os.path.join(self.configdir, "mock-%s.cfg"
-                                   % str(threading.current_thread().name))
+        mock_config = os.path.join(
+            self.configdir, "mock-%s.cfg" % str(threading.current_thread().name))
 
         # Get the build-id in thread-safe manner.
         build_id = None
@@ -457,19 +542,18 @@ class MockModuleBuilder(GenericBuilder):
 
         # Clear resultsdir associated with this thread or in case it does not
         # exist, create it.
-        resultsdir = os.path.join(self.resultsdir,
-                                  str(threading.current_thread().name))
+        resultsdir = os.path.join(self.resultsdir, str(threading.current_thread().name))
         if os.path.exists(resultsdir):
             for name in os.listdir(resultsdir):
                 os.remove(os.path.join(resultsdir, name))
         else:
             os.makedirs(resultsdir)
 
-        # Git sources are treated specially.
-        if source.startswith(("git://", "http://", "https://")):
-            builder = SCMBuilder(mock_config, resultsdir, source, artifact_name)
+        if source.endswith(".src.rpm"):
+            builder = SRPMBuilder(mock_config, resultsdir, source)
         else:
-            builder = LocalBuilder(mock_config, resultsdir, source)
+            # Otherwise, assume we're building from some scm repo
+            builder = SCMBuilder(mock_config, resultsdir, source, artifact_name)
         return self.build_srpm(artifact_name, source, build_id, builder)
 
     @staticmethod
@@ -480,25 +564,56 @@ class MockModuleBuilder(GenericBuilder):
     def cancel_build(self, task_id):
         pass
 
-    def list_tasks_for_components(self, component_builds=None, state='active'):
+    def list_tasks_for_components(self, component_builds=None, state="active"):
         pass
 
+    def repo_from_tag(cls, config, tag_name, arch):
+        pass
+
+    def finalize(self, succeeded=True):
+        # For successful builds, do one last createrepo, to include
+        # the module metadata. We don't want to do this for failed builds,
+        # since that makes it impossible to retry a build manually.
+        if succeeded:
+            self._createrepo(include_module_yaml=True)
+
+    @classmethod
+    def get_built_rpms_in_module_build(cls, mmd):
+        """
+        :param Modulemd mmd: Modulemd to get the built RPMs from.
+        :return: list of NVRs
+        """
+        with models.make_db_session(conf) as db_session:
+            build = models.ModuleBuild.get_build_from_nsvc(
+                db_session,
+                mmd.get_module_name(),
+                mmd.get_stream_name(),
+                mmd.get_version(),
+                mmd.get_context()
+            )
+            if build.koji_tag.startswith("repofile://"):
+                # Modules from local repository have already the RPMs filled in mmd.
+                return mmd.get_rpm_artifacts()
+            else:
+                koji_session = KojiModuleBuilder.get_session(conf, login=False)
+                rpms = koji_session.listTaggedRPMS(build.koji_tag, latest=True)[0]
+                nvrs = set(kobo.rpmlib.make_nvr(rpm, force_epoch=True) for rpm in rpms)
+                return list(nvrs)
+
 
 class BaseBuilder(object):
     def __init__(self, config, resultsdir):
         self.config = config
         self.resultsdir = resultsdir
-        self.cmd = ["mock", "-v", "-r", config,
-                    "--no-clean",
-                    "--resultdir=%s" % resultsdir]
+        self.cmd = ["mock", "-v", "-r", config, "--no-clean", "--resultdir=%s" % resultsdir]
 
     def build(self, stdout, stderr):
         execute_cmd(self.cmd, stdout=stdout, stderr=stderr)
 
 
-class LocalBuilder(BaseBuilder):
+class SRPMBuilder(BaseBuilder):
     def __init__(self, config, resultsdir, source):
-        super(LocalBuilder, self).__init__(config, resultsdir)
+        super(SRPMBuilder, self).__init__(config, resultsdir)
         self.cmd.extend(["--rebuild", source])
 
 
@@ -506,9 +621,10 @@ class SCMBuilder(BaseBuilder):
     def __init__(self, config, resultsdir, source, artifact_name):
         super(SCMBuilder, self).__init__(config, resultsdir)
         with open(config, "a") as f:
-            branch = source.split("?#")[1]
+            repo_path, branch = source.split("?#")
             distgit_cmds = self._get_distgit_commands(source)
-            distgit_get = distgit_cmds[0].format(artifact_name)
+            # Supply the artifact name for "{0}" and the full path to the repo for "{repo_path}"
+            distgit_get = distgit_cmds[0].format(artifact_name, repo_path=repo_path)
 
             # mock-scm cannot checkout particular commit hash, but only branch.
             # We therefore use a command that combines the distgit-command with
@@ -516,25 +632,34 @@ class SCMBuilder(BaseBuilder):
             # See https://bugzilla.redhat.com/show_bug.cgi?id=1459437 for
             # more info. Once mock-scm supports this feature, we can remove
             # this code.
-            distgit_get_branch = \
-                "sh -c {}'; git -C {} checkout {}'".format(pipes.quote(distgit_get),
-                                                           artifact_name,
-                                                           branch)
+            distgit_get_branch = "sh -c {}'; git -C {} checkout {}'".format(
+                pipes.quote(distgit_get), artifact_name, branch)
 
             f.writelines([
                 "config_opts['scm'] = True\n",
                 "config_opts['scm_opts']['method'] = 'distgit'\n",
-                "config_opts['scm_opts']['package'] = '{}'\n".format(
-                    artifact_name),
-                "config_opts['scm_opts']['distgit_get'] = {!r}\n".format(
-                    distgit_get_branch),
-                "config_opts['scm_opts']['distgit_src_get'] = '{}'\n".format(
-                    distgit_cmds[1]),
+                "config_opts['scm_opts']['package'] = '{}'\n".format(artifact_name),
+                "config_opts['scm_opts']['distgit_get'] = {!r}\n".format(distgit_get_branch),
             ])
 
+            # Set distgit_src_get only if it's defined.
+            if distgit_cmds[1]:
+                f.write(
+                    "config_opts['scm_opts']['distgit_src_get'] = '{}'\n".format(distgit_cmds[1]))
+
+            # The local git repositories cloned by `fedpkg clone` typically do not have
+            # the tarballs with sources committed in a git repo. They normally live in lookaside
+            # cache on remote server, but we should not try getting them from there for true
+            # local builds.
+            # Instead, get them from local path with git repository by passing that path to Mock
+            # using the `ext_src_dir`.
+            if repo_path.startswith("file://"):
+                src_dir = repo_path[len("file://"):]
+                f.write("config_opts['scm_opts']['ext_src_dir'] = '{}'\n".format(src_dir))
+
     def _make_executable(self, path):
         mode = os.stat(path).st_mode
-        mode |= (mode & 0o444) >> 2    # copy R bits to X
+        mode |= (mode & 0o444) >> 2  # copy R bits to X
         os.chmod(path, mode)
 
     def _get_distgit_commands(self, source):
@@ -554,11 +679,11 @@ class SCMBuilder(BaseBuilder):
         if not self.koji_session:
             # If Koji is not configured on the system, then just return 0.0 for components
             try:
-                self.koji_session = KojiModuleBuilder.get_session(self.config, self.owner)
+                self.koji_session = KojiModuleBuilder.get_session(self.config, login=False)
                 # If the component has not been built before, then None is returned. Instead,
                 # let's return 0.0 so the type is consistent
                 return self.koji_session.getAverageBuildDuration(component.package) or 0.0
             except Exception:
-                log.debug('The Koji call to getAverageBuildDuration failed. Is Koji properly '
-                          'configured?')
+                log.debug(
+                    "The Koji call to getAverageBuildDuration failed. Is Koji properly configured?")
                 return 0.0
