@@ -34,35 +34,47 @@ import tempfile
 import shutil
 import datetime
 
-from module_build_service import log
+from module_build_service import log, conf
 from module_build_service.errors import (
-    Forbidden, ValidationError, UnprocessableEntity, ProgrammingError)
-import module_build_service.utils
+    Forbidden,
+    ValidationError,
+    UnprocessableEntity,
+    ProgrammingError,
+)
+from module_build_service.utils.general import scm_url_schemes, retry
 
 
 class SCM(object):
     "SCM abstraction class"
 
     # Assuming git for HTTP schemas
-    types = module_build_service.utils.scm_url_schemes()
+    types = scm_url_schemes()
 
     def __init__(self, url, branch=None, allowed_scm=None, allow_local=False):
-        """Initialize the SCM object using the specified scmurl.
+        """
+        Initialize the SCM object using the specified SCM URL.
 
         If url is not in the list of allowed_scm, an error will be raised.
 
         :param str url: The unmodified scmurl
-        :param list allowed_scm: The list of allowed SCMs, optional
+        :param str branch: The optional source control branch. This defaults to "master" when git
+            is used.
+        :param list allowed_scm: The optional list of allowed SCM URL prefixes
+        :param bool allow_local: Allow SCM URLs that start with "file://"
         :raises: Forbidden or ValidationError
         """
 
         if allowed_scm:
-            if not (url.startswith(tuple(allowed_scm)) or
-                    (allow_local and url.startswith("file://"))):
-                raise Forbidden(
-                    '%s is not in the list of allowed SCMs' % url)
+            if not (
+                url.startswith(tuple(allowed_scm)) or (allow_local and url.startswith("file://"))
+            ):
+                raise Forbidden("%s is not in the list of allowed SCMs" % url)
 
-        url = url.rstrip('/')
+        # If we are given the option for the git protocol or the http(s) protocol,
+        # then just use http(s)
+        if re.match(r"(git\+http(?:s)?:\/\/)", url):
+            url = url[4:]
+        url = url.rstrip("/")
 
         self.url = url
         self.sourcedir = None
@@ -74,20 +86,18 @@ class SCM(object):
                 self.scheme = scmtype
                 break
         else:
-            raise ValidationError('Invalid SCM URL: %s' % url)
+            raise ValidationError("Invalid SCM URL: %s" % url)
 
         # git is the only one supported SCM provider atm
         if self.scheme == "git":
             match = re.search(r"^(?P<repository>.*/(?P<name>[^?]*))(\?#(?P<commit>.*))?", url)
             self.repository = match.group("repository")
             self.name = match.group("name")
-            self.repository_root = self.repository[:-len(self.name)]
+            self.repository_root = self.repository[: -len(self.name)]
             if self.name.endswith(".git"):
                 self.name = self.name[:-4]
             self.commit = match.group("commit")
             self.branch = branch if branch else "master"
-            if not self.commit:
-                self.commit = self.get_latest(self.branch)
             self.version = None
         else:
             raise ValidationError("Unhandled SCM scheme: %s" % self.scheme)
@@ -104,9 +114,10 @@ class SCM(object):
             raise ProgrammingError("Do .checkout() first.")
 
         found = False
-        branches = SCM._run(["git", "branch", "-r", "--contains", self.commit],
-                            chdir=self.sourcedir)[1]
-        for branch in branches.split("\n"):
+        branches = SCM._run(
+            ["git", "branch", "-r", "--contains", self.commit], chdir=self.sourcedir
+        )[1]
+        for branch in branches.decode("utf-8").split("\n"):
             branch = branch.strip()
             if branch[len("origin/"):] == self.branch:
                 found = True
@@ -125,8 +136,7 @@ class SCM(object):
         return None
 
     @staticmethod
-    @module_build_service.utils.retry(wait_on=RuntimeError)
-    def _run(cmd, chdir=None, log_stdout=False):
+    def _run_without_retry(cmd, chdir=None, log_stdout=False):
         proc = sp.Popen(cmd, stdout=sp.PIPE, stderr=sp.PIPE, cwd=chdir)
         stdout, stderr = proc.communicate()
         if log_stdout and stdout:
@@ -134,10 +144,20 @@ class SCM(object):
         if stderr:
             log.warning(stderr)
         if proc.returncode != 0:
-            raise UnprocessableEntity("Failed on %r, retcode %r, out %r, err %r" % (
-                cmd, proc.returncode, stdout, stderr))
+            raise UnprocessableEntity(
+                "Failed on %r, retcode %r, out %r, err %r" % (cmd, proc.returncode, stdout, stderr)
+            )
         return proc.returncode, stdout, stderr
 
+    @staticmethod
+    @retry(
+        timeout=conf.scm_net_timeout,
+        interval=conf.scm_net_retry_interval,
+        wait_on=UnprocessableEntity,
+    )
+    def _run(cmd, chdir=None, log_stdout=False):
+        return SCM._run_without_retry(cmd, chdir, log_stdout)
+
     def checkout(self, scmdir):
         """Checkout the module from SCM.
 
@@ -147,29 +167,26 @@ class SCM(object):
         """
         # TODO: sanity check arguments
         if self.scheme == "git":
-            self.sourcedir = '%s/%s' % (scmdir, self.name)
-
-            module_clone_cmd = ['git', 'clone', '-q']
-            if self.commit:
-                module_checkout_cmd = ['git', 'checkout', '-q', self.commit]
-            else:
-                module_clone_cmd.extend(['--depth', '1'])
-            module_clone_cmd.extend([self.repository, self.sourcedir])
+            self.sourcedir = "%s/%s" % (scmdir, self.name)
 
+            module_clone_cmd = [
+                "git", "clone", "-q", "--no-checkout", self.repository, self.sourcedir
+            ]
+            module_checkout_cmd = ["git", "checkout", "-q", self.commit]
             # perform checkouts
             SCM._run(module_clone_cmd, chdir=scmdir)
-            if self.commit:
-                try:
-                    SCM._run(module_checkout_cmd, chdir=self.sourcedir)
-                except RuntimeError as e:
-                    if (e.message.endswith(
-                            " did not match any file(s) known to git.\\n\"") or
-                       "fatal: reference is not a tree: " in e.message):
-                        raise UnprocessableEntity(
-                            "checkout: The requested commit hash was not found "
-                            "within the repository. Perhaps you forgot to push. "
-                            "The original message was: %s" % e.message)
-                    raise
+            try:
+                SCM._run(module_checkout_cmd, chdir=self.sourcedir)
+            except RuntimeError as e:
+                if (
+                    e.message.endswith(' did not match any file(s) known to git.\\n"')
+                    or "fatal: reference is not a tree: " in e.message
+                ):
+                    raise UnprocessableEntity(
+                        "checkout: The requested commit hash was not found within the repository. "
+                        "Perhaps you forgot to push. The original message was: %s" % e.message
+                    )
+                raise
 
             timestamp = SCM._run(["git", "show", "-s", "--format=%ct"], chdir=self.sourcedir)[1]
             dt = datetime.datetime.utcfromtimestamp(int(timestamp))
@@ -178,22 +195,37 @@ class SCM(object):
             raise RuntimeError("checkout: Unhandled SCM scheme.")
         return self.sourcedir
 
-    def get_latest(self, branch='master'):
-        """Get the latest commit ID.
+    def get_latest(self, ref=None):
+        """ Get the latest commit hash based on the provided git ref
 
-        :returns: str -- the latest commit ID, e.g. the git $BRANCH HEAD
+        :param ref: a string of a git ref (either a branch or commit hash). This defaults to
+            self.branch.
+        :returns: a string of the latest commit hash
         :raises: RuntimeError
         """
+        if ref is None:
+            ref = self.branch
+
         if self.scheme == "git":
             log.debug("Getting/verifying commit hash for %s" % self.repository)
-            output = SCM._run(["git", "ls-remote", "--exit-code", self.repository, branch])[1]
-            if output:
-                self.commit = output.split("\t")[0]
-                return self.commit
-
-            # Hopefully `branch` is really a commit hash.  Code later needs to verify this.
-            if self.is_available(True):
-                return branch
+            try:
+                # This will fail if `ref` is not a branch name, but this works for commit hashes.
+                # If the ref is not a branch, then fallback to `get_full_commit_hash`. We do not
+                # want to retry here, since if it's a commit, it would block for a very long time.
+                cmd = ["git", "ls-remote", "--exit-code", self.repository, "refs/heads/" + ref]
+                log.debug("Checking to see if the ref %s is a branch with `%s`", ref, " ".join(cmd))
+                _, output, _ = SCM._run_without_retry(cmd)
+            except UnprocessableEntity:
+                log.debug("The ref %s is not a branch. Checking to see if it's a commit hash", ref)
+                # The call below will either return the commit hash as is (if a full one was
+                # provided) or the full commit hash (if a short hash was provided). If ref is not
+                # a commit hash, then this will raise an exception.
+                return self.get_full_commit_hash(commit_hash=ref)
+            else:
+                # git-ls-remote prints output like this, where the first commit
+                # hash is what to return.
+                # bf028e573e7c18533d89c7873a411de92d4d913e	refs/heads/master
+                return output.split()[0].decode("utf-8")
         else:
             raise RuntimeError("get_latest: Unhandled SCM scheme.")
 
@@ -206,33 +238,46 @@ class SCM(object):
         """
         if commit_hash:
             commit_to_check = commit_hash
-        elif self.commit:
-            commit_to_check = self.commit
+        elif self._commit:
+            commit_to_check = self._commit
         else:
-            raise RuntimeError('No commit hash was specified for "{0}"'.format(
-                self.url))
+            try:
+                # If self._commit was None, then calling `self.commit` will resolve the ref based
+                # on the branch
+                return self.commit
+            except UnprocessableEntity:
+                # If there was an exception resolving the ref based on the branch (could be the
+                # default branch that doesn't exist), then there is not enough information to get
+                # the commit hash
+                raise RuntimeError('No commit hash was specified for "{0}"'.format(self.url))
 
-        if self.scheme == 'git':
-            log.debug('Getting the full commit hash for "{0}"'
-                      .format(self.repository))
+        if self.scheme == "git":
+            log.debug(
+                "Getting the full commit hash on %s from %s", self.repository, commit_to_check)
             td = None
             try:
                 td = tempfile.mkdtemp()
-                SCM._run(['git', 'clone', '-q', self.repository, td])
-                output = SCM._run(
-                    ['git', 'rev-parse', commit_to_check], chdir=td)[1]
+                SCM._run(["git", "clone", "-q", self.repository, td, "--bare"])
+                cmd = ["git", "rev-parse", commit_to_check]
+                log.debug(
+                    "Running `%s` to get the full commit hash for %s",
+                    " ".join(cmd),
+                    commit_to_check
+                )
+                output = SCM._run(cmd, chdir=td)[1]
             finally:
                 if td and os.path.exists(td):
                     shutil.rmtree(td)
 
             if output:
-                return str(output.strip('\n'))
+                return str(output.decode("utf-8").strip("\n"))
 
-            raise RuntimeError(
-                'The full commit hash of "{0}" for "{1}" could not be found'
-                .format(commit_hash, self.repository))
+            raise UnprocessableEntity(
+                'The full commit hash of "{0}" for "{1}" could not be found'.format(
+                    commit_hash, self.repository)
+            )
         else:
-            raise RuntimeError('get_full_commit_hash: Unhandled SCM scheme.')
+            raise RuntimeError("get_full_commit_hash: Unhandled SCM scheme.")
 
     def get_module_yaml(self):
         """
@@ -249,9 +294,11 @@ class SCM(object):
             with open(path_to_yaml):
                 return path_to_yaml
         except IOError:
-            raise UnprocessableEntity(
-                "get_module_yaml: SCM repository doesn't seem to contain a "
-                "module YAML file. Couldn't access: %s" % path_to_yaml)
+            log.error(
+                "get_module_yaml: The SCM repository doesn't contain a modulemd file. "
+                "Couldn't access: %s" % path_to_yaml
+            )
+            raise UnprocessableEntity("The SCM repository doesn't contain a modulemd file")
 
     @staticmethod
     def is_full_commit_hash(scheme, commit):
@@ -262,87 +309,20 @@ class SCM(object):
         :param commit: a string containing the commit
         :return: boolean
         """
-        if scheme == 'git':
-            sha1_pattern = re.compile(r'^[0-9a-f]{40}$')
+        if scheme == "git":
+            sha1_pattern = re.compile(r"^[0-9a-f]{40}$")
             return bool(re.match(sha1_pattern, commit))
         else:
-            raise RuntimeError('is_full_commit_hash: Unhandled SCM scheme.')
-
-    def is_available(self, strict=False):
-        """Check whether the scmurl is available for checkout.
-
-        :param bool strict: When True, raise expection on error instead of
-                            returning False.
-        :returns: bool -- the scmurl is available for checkout
-        """
-        td = None
-        try:
-            td = tempfile.mkdtemp()
-            self.checkout(td)
-            return True
-        except Exception:
-            if strict:
-                raise
-            return False
-        finally:
-            try:
-                if td is not None:
-                    shutil.rmtree(td)
-            except Exception as e:
-                log.warning(
-                    "Failed to remove temporary directory {!r}: {}".format(
-                        td, str(e)))
-
-    @property
-    def url(self):
-        """The original scmurl."""
-        return self._url
-
-    @url.setter
-    def url(self, s):
-        self._url = str(s)
-
-    @property
-    def scheme(self):
-        """The SCM scheme."""
-        return self._scheme
-
-    @scheme.setter
-    def scheme(self, s):
-        self._scheme = str(s)
-
-    @property
-    def sourcedir(self):
-        """The SCM source directory."""
-        return self._sourcedir
-
-    @sourcedir.setter
-    def sourcedir(self, s):
-        self._sourcedir = str(s)
-
-    @property
-    def repository(self):
-        """The repository part of the scmurl."""
-        return self._repository
-
-    @repository.setter
-    def repository(self, s):
-        self._repository = str(s)
+            raise RuntimeError("is_full_commit_hash: Unhandled SCM scheme.")
 
     @property
     def commit(self):
         """The commit ID, for example the git hash, or None."""
+        if not self._commit:
+            self._commit = self.get_latest(self.branch)
+
         return self._commit
 
     @commit.setter
     def commit(self, s):
         self._commit = str(s) if s else None
-
-    @property
-    def name(self):
-        """The module name."""
-        return self._name
-
-    @name.setter
-    def name(self, s):
-        self._name = str(s)
